#!/bin/sh
#
# PVR HW-recovery dumper.
#
# This tool dumps all the state provided by the pvr drivers hwrec interface.
#
# It reads debugfs/pvr/hwrec_event, which blocks until a hwrec event happens.
# Then it tars up hwrec_regs, hwrec_status, hwrec_mem and hwrec_edm and
# lzops it for later retrieval and analysis.
#
# Intended use case: call this script from /etc/init/sgx.conf after
# modprobe omaplfb, so that hwrec dumps are provided all the time.
#

set -e

#
# Edit these.
#
DEBUGFS_DIR=/sys/kernel/debug
DUMP_BASE_DIR=/home/user/MyDocs
DUMP_DIR=$DUMP_BASE_DIR/pvr_hwrec
DUMP_TMP_DIR=$DUMP_BASE_DIR/pvr_hwrec_tmp
DUMP_FILE_STEM=pvr_hwrec
TMP_DIR=/tmp
RICH_CORE_WATCH_PROC=pvrsrv-trigger-core
RICH_CORE_CONFIG=/etc/rich-core/$RICH_CORE_WATCH_PROC.extras

# $DUMP_DIR can be on an FS that doesn't support links, so put
# the following to /tmp instead.
LAST_DUMP_FILE=$TMP_DIR/pvr_hwrec_last.tar.lzo
MAX_LOG_FILE_CNT=3

PVR_DEBUGFS_DIR=$DEBUGFS_DIR/pvr/

if [ ! -d $TMP_DIR ]; then
	echo "$TMP_DIR doesn't exist!"
	exit
fi

if [ ! -e $RICH_CORE_CONFIG ] ; then
	echo "Can't read rich core config file $RICH_CORE_CONFIG!"
	exit
fi

RICH_CORE_FILE="$( cat $RICH_CORE_CONFIG | tr -d '\n ' )"

if [ "$RICH_CORE_FILE" != "$LAST_DUMP_FILE" ] ; then
	echo "Rich core config path '$RICH_CORE_FILE' should be '$LAST_DUMP_FILE'!"
	exit
fi

if [ `id -nu` != "root" ]; then
	echo "dumper needs to be run as root!"
	exit
fi

# mount debugfs if needed.
if [ ! -e $DEBUGFS_DIR ]; then
	echo "Debugfs mount point $DEBUGFS_DIR doesn't exist!"
	exit
fi

if [ ! -e $PVR_DEBUGFS_DIR ]; then
	mount -t debugfs none $DEBUGFS_DIR
fi

if [ ! -e $PVR_DEBUGFS_DIR ]; then
	echo "Failed to mount debugfs!"
	exit
fi

# check debugfs hwrec files.
if [ ! -e $PVR_DEBUGFS_DIR/hwrec_event -o \
     ! -e $PVR_DEBUGFS_DIR/hwrec_time -o \
     ! -e $PVR_DEBUGFS_DIR/hwrec_regs -o \
     ! -e $PVR_DEBUGFS_DIR/hwrec_status ]; then
	echo "Failed to find debugfs hwrec files!"
	exit
fi

if [ ! -f $PVR_DEBUGFS_DIR/hwrec_event -o \
     ! -f $PVR_DEBUGFS_DIR/hwrec_time -o \
     ! -f $PVR_DEBUGFS_DIR/hwrec_regs -o \
     ! -f $PVR_DEBUGFS_DIR/hwrec_status -o \
     ! -r $PVR_DEBUGFS_DIR/hwrec_event -o \
     ! -r $PVR_DEBUGFS_DIR/hwrec_time -o \
     ! -r $PVR_DEBUGFS_DIR/hwrec_regs -o \
     ! -r $PVR_DEBUGFS_DIR/hwrec_status ]; then
	echo "Unable to access debugfs hwrec files!"
	exit
fi

# might have been built in conditionally.
if [ ! -e $PVR_DEBUGFS_DIR/hwrec_edm -o \
     ! -f $PVR_DEBUGFS_DIR/hwrec_edm -o \
     ! -r $PVR_DEBUGFS_DIR/hwrec_edm ]; then
	HAVE_EDM=
else
	HAVE_EDM=1
fi

if [ ! -e $PVR_DEBUGFS_DIR/hwrec_mem -o \
     ! -f $PVR_DEBUGFS_DIR/hwrec_mem -o \
     ! -r $PVR_DEBUGFS_DIR/hwrec_mem ]; then
	HAVE_MEM=
else
	HAVE_MEM=1
fi

echo "creating $DUMP_DIR"
mkdir -p $DUMP_DIR

dump_logs()
{
	local dump_file

	dump_file="$1"

	rm -rf $DUMP_TMP_DIR/*
	mkdir -p $DUMP_TMP_DIR

	cat $PVR_DEBUGFS_DIR/hwrec_regs > $DUMP_TMP_DIR/hwrec_regs
	cat $PVR_DEBUGFS_DIR/hwrec_status > $DUMP_TMP_DIR/hwrec_status

	if [ $HAVE_EDM ]; then
		cat $PVR_DEBUGFS_DIR/hwrec_edm > $DUMP_TMP_DIR/hwrec_edm
	fi

	if [ $HAVE_MEM ]; then
		cat $PVR_DEBUGFS_DIR/hwrec_mem > $DUMP_TMP_DIR/hwrec_mem
	fi

	tar -c $DUMP_TMP_DIR | lzop > $dump_file

	rm -rf $DUMP_TMP_DIR
}

delete_old_dumps()
{
	local dump_file_stem

	dump_file_stem="$1"

	# ls fails if there are no dumps
	set +e
	old_dumps=$( ls -t $dump_file_stem* 2> /dev/null | sed 1,${MAX_LOG_FILE_CNT}d )
	set -e

	rm -rf $old_dumps
}

wakeup_rich_core()
{
	# The only robust way to wake up the rich core tool is to
	# generate a core dump and let the kernel call the tool through
	# /proc/sys/kernel/core_pattern. Thus we don't need to care
	# about any race with any ongoing core dumping and we provide
	# the core-matic server with a valid input, which must include a
	# process core dump section.

	set +e
	/usr/sbin/pvrsrv-trigger-core
	set -e
}

# Now do the actual work.
while true
do
	cat $PVR_DEBUGFS_DIR/hwrec_event

	TIMESTAMP=`cat $PVR_DEBUGFS_DIR/hwrec_time`
	DUMP_STEM=$DUMP_DIR/$DUMP_FILE_STEM
	DUMP_FILE=${DUMP_STEM}_$TIMESTAMP.tar.lzo

	if [ -e $DUMP_FILE ]; then
		echo "Skipping dump: $DUMP_FILE already exists."
		continue
	fi

	dump_logs "$DUMP_FILE"
	delete_old_dumps "$DUMP_STEM"
	ln -sf $DUMP_FILE $LAST_DUMP_FILE

	wakeup_rich_core

	echo "Dumped HWRecovery frame to $DUMP_FILE"
done
